/*
  Power Remote Display V9.0_DISP
  *******************************

  Affiche la consommation  ou la surproduction de la maison en W et  Wh
  Capture les données du routeur ESP32  de F1ATB

  Versions 
  ********  
  Version 7.00_DISP Affiche la couleur des contrats Tempo
  Version 8.00_DISP  prends en compte les mots de passe avec caratères spéciaux pour le Wifi
  Version 9.00_DISP  est adapté pour le routeur photovoltaique version V9.xx
  
  Détails sur :
  https://f1atb.fr  Section Domotique 

  F1ATB Juin 2024
  GNU General Public License v3.0

*/

#define Version "9.00_DISP"
#define HOSTNAME "DISP-ESP32-"
#define CLE_Rom_Init 721234807


#define Taille_data 22

//Libraries
//TFT Display
#include <TFT_eSPI.h>  // Graphics and font library for ST7735 driver chip
#include <SPI.h>
#include "Free_Fonts.h"
#include "EEPROM.h"
#include <esp_task_wdt.h>  //Pour un Watchdog


//WIFI
#include <WiFi.h>
#include <WiFiClient.h>
#include <ESPmDNS.h>
#include <WebServer.h>
#include <ArduinoOTA.h>   //Modification On The Air
#include <ESPmDNS.h>

//Program routines
#include "pageHtmlConnect.h"
#include "pageHtmlMain.h"
#include "pageHtmlPara.h"

//Watchdog de 1800 secondes. Le systeme se Reset si pas de dialoque avec la source
#define WDT_TIMEOUT 1800

unsigned long RMSextIP = 0;
String IPext = "0.0.0.0";
const int httpPort = 80;

//WIFI
String ssid = "";
String password = "";
const char *ap_default_ssid;        // Mode Access point  IP: 192.168.4.1
const char *ap_default_psk = NULL;  // Pas de mot de passe en AP,
String hostname(HOSTNAME);


//PINS - GPIO
const int GPIO0 = 0;     //Bouton
const int GPIO35 = 35;   //Bouton
const int GPIO_IR = 33;  //Détecteur présence

//VARIABLES


int But0 = 1;
int But35 = 1;
bool AfficheurOn = true;
bool GoodIRpulse = true;
bool EnergieActiveValide = false;
String Heure;
int min10 = 0;                 //Compteur diaine minutes
String GS = String((char)29);  //Group Separator
String RS = String((char)30);  //Record Separator
String Source_data;
String LTARF = "";  //Option Tarifaire du Linky
String STGE = "0";  //Option Tarifaire jour et lendemain etc en hexa
float Temperature = -125;
unsigned long Cle_ROM;
String data_[Taille_data];
int Imax = -1;
int Bug_Host = 0;

//Paramètres électriques
float Tension_T, Intensite_T, PowerFactor_T, Frequence;
float Tension_M, Intensite_M, PowerFactor_M;
long Energie_T_Soutiree = 0;
long Energie_T_Injectee = 0;
long Energie_M_Soutiree = 0;
long Energie_M_Injectee = 0;
long EnergieJour_T_Injectee = 0;
long EnergieJour_M_Injectee = 0;
long EnergieJour_T_Soutiree = 0;
long EnergieJour_M_Soutiree = 0;
int PuissanceS_T, PuissanceS_M, PuissanceI_T, PuissanceI_M;
int PVAS_T, PVAS_M, PVAI_T, PVAI_M;
int Pva_valide;

//Variable sauvegardée pendant le Deep Sleep
RTC_DATA_ATTR int LastW = 0;
RTC_DATA_ATTR int idxDisp = 0;
RTC_DATA_ATTR int idxPw = 0;
RTC_DATA_ATTR int tabPw[240];  //tableau des puissances sur 40h


//Internal Timers
unsigned long startMillis;
unsigned long previousDataMillis;
unsigned long previousIRMillis;
unsigned long dureeAllumageMillis;

//TFT
TFT_eSPI tft = TFT_eSPI();  // Invoke library, pins defined in User_Setup_Select.h
// WEB Server
WebServer server(80);  // Simple Web Server on port 80

// SETUP
//*******
void setup() {
  esp_sleep_wakeup_cause_t source_reveil;
  source_reveil = esp_sleep_get_wakeup_cause();  //0= boot, 2=interruption capteur IR, 4 = interruption timer. Valeurs différentes de la doc Espressif ?
  AfficheurOn = true;
  dureeAllumageMillis = 10000;  //10s
  if (source_reveil == 4) {
    AfficheurOn = false;
  }
  if (source_reveil == 0) {
    //Watchdog initialisation
    esp_task_wdt_init(WDT_TIMEOUT, true);  //enable panic so ESP32 restarts
  }

  startMillis = millis();
  Serial.begin(115200);
  Serial.println("Booting");
  tft.init();
  tft.setRotation(1);
  tft.setTextColor(TFT_WHITE, TFT_NAVY);
  Serial.println("Booting");
  tft.println("Booting");
  delay(1000);
  Serial.println("Wake up : " + String(source_reveil));
  int IR = digitalRead(GPIO_IR);
  Serial.println("IR at wake up : " + String(IR));
  if (AfficheurOn) TraceWatt();

  // Configure WIFI
  // **************

  hostname += String(ESP.getEfuseMac() & 0xFFF);
  WiFi.hostname(hostname);
  Serial.println(hostname);
  ap_default_ssid = (const char *)hostname.c_str();
  // Check WiFi connection
  // ... check mode
  if (WiFi.getMode() != WIFI_STA) {
    WiFi.mode(WIFI_STA);
    delay(10);
  }
  INIT_EEPROM();
  Cle_ROM = CLE_Rom_Init;
  unsigned long Rcle = LectureCle();
  Serial.println("cle : " + String(Rcle));
  if (Rcle == Cle_ROM) {  // Programme déjà executé
    LectureSsidPasse();
  } else {
  }

  //WIFI
  if (ssid.length() > 0) {
    Serial.println("SSID:" + ssid);
    Serial.println("Pass:" + password);
    Serial.println("Wifi Begin : " + ssid);
    WiFi.begin(ssid.c_str(), password.c_str());
    while (WiFi.status() != WL_CONNECTED && (millis() - startMillis < 15000)) {  // Attente connexion au Wifi
      Serial.write('.');
      Serial.print(WiFi.status());
      delay(300);
    }
  }
  if (WiFi.status() == WL_CONNECTED) {
    MDNS.begin(hostname);
    Serial.print("IP address: ");
    Serial.println(WiFi.localIP());
    Serial.println("Connected IP address: " + WiFi.localIP().toString() + " or <a href='http://" + hostname + ".local' >" + hostname + ".local</a>");
  } else {
    Serial.println("Can not connect to WiFi station. Go into AP mode and STA mode.");
    // Go into software AP and STA modes.
    WiFi.disconnect();
    WiFi.mode(WIFI_AP_STA);
    delay(10);
    WiFi.softAP(ap_default_ssid, ap_default_psk);
    Serial.print("Access Point Mode. IP address: ");
    Serial.println(WiFi.softAPIP());
    dureeAllumageMillis = 200000;  //200s
  }

 
  Init_Server();
  //Various Timers

  previousDataMillis = 0;
  previousIRMillis = millis();



  pinMode(GPIO0, INPUT);
  pinMode(GPIO35, INPUT);
  pinMode(GPIO_IR, INPUT_PULLUP);

  // Modification du programme par le Wifi  - OTA(On The Air)
  //***************************************************
  ArduinoOTA.setHostname((const char *)hostname.c_str());
  ArduinoOTA.begin();  //Mandatory

  //2 modes de Deep Sleep
  esp_sleep_enable_ext0_wakeup(GPIO_NUM_33, 1);  //Réveil si presence Infra Rouge passe à 1
  esp_sleep_enable_timer_wakeup(590000000);      //Réveil périodique en uS sans affichage pour avoir immediatement un résultat à afficher lors d'un reveil IR
  IR = digitalRead(GPIO_IR);
  Serial.println("IR à la fin du setup : " + String(IR));
  GoodIRpulse = true;
  if (source_reveil == 4 && IR == 1) GoodIRpulse = false;  //IR déclenché par le wifi. Perturbe la lecture de la puissance en veille écran eteint
  esp_task_wdt_reset();
}

// LOOP
//******
void loop() {
  //Gestion des serveurs
  //********************
  ArduinoOTA.handle();
  server.handleClient();

  if (digitalRead(GPIO0) != But0) {  //Nouvelle page demandée
    But0 = digitalRead(GPIO0);
    if (But0 == 0) {
      idxDisp = (idxDisp + 1) % 4;
      tft.fillScreen(TFT_DARKGREY);
      Affichage();
      previousDataMillis = millis() - 2000;  // Accélerer l'appel au serveur de données
      previousIRMillis = millis();
      dureeAllumageMillis = 300000;
    }
  }
  if (digitalRead(GPIO35) != But35) {  //Nouvelle page demandée
    But35 = digitalRead(GPIO35);
    if (But35 == 0) {
      idxDisp = (idxDisp + 3) % 4;
      tft.fillScreen(TFT_DARKGREY);
      Affichage();
      previousDataMillis = millis() - 2000;  // Accélerer l'appel au serveur de données
      previousIRMillis = millis();
      dureeAllumageMillis = 300000;
    }
  }

  LireIR();

  if ((millis() - previousDataMillis >= 2000)) {
    int IR = digitalRead(GPIO_IR);

    Serial.println("IR loop : " + String(IR));
    Lecture_Puissance();  //Call Power Router for last data
    Serial.println(String(LastW) + " W");
    //Rafraichissement Affichage
    if (AfficheurOn && GoodIRpulse) Affichage();
    previousDataMillis = millis();
  }

  delay(10);
}



// Power Router client
//********************
//Recherche de la puissance auprès du routeur solaire de F1ATB
void Lecture_Puissance() {
  String url;
  int p;
  int min10new;
  String RMSExtDataB = "";
  String Gr[4];

  // Use WiFiClient class to create TCP connections
  WiFiClient clientESP_RMS;
  byte arr[4];
  arr[0] = RMSextIP & 0xFF;          // 0x78
  arr[1] = (RMSextIP >> 8) & 0xFF;   // 0x56
  arr[2] = (RMSextIP >> 16) & 0xFF;  // 0x34
  arr[3] = (RMSextIP >> 24) & 0xFF;  // 0x12

  IPext = String(arr[3]) + "." + String(arr[2]) + "." + String(arr[1]) + "." + String(arr[0]);
  if (!clientESP_RMS.connect(IPext.c_str(), httpPort)) {
    Serial.println("Connexion vers " + String(IPext) + " échoué. Nb bug : " + String(Bug_Host));
 
    Bug_Host++;
    if (Bug_Host > 100) {
      ESP.restart();
    }
    clientESP_RMS.stop();
    delay(500);
    return;
  }
  // We now create a URI for the request
  url = "/ajax_data";

  clientESP_RMS.print(String("GET ") + url + " HTTP/1.1\r\n" + "Host: " + IPext + "\r\n" + "Connection: close\r\n\r\n");
  unsigned long timeout = millis();
  while (clientESP_RMS.available() == 0) {
    if (millis() - timeout > 5000) {
      Serial.println(">>> Client Timeout IPext : " + String(IPext));
      tft.println(">>> Client Timeout ! IPext : " + String(IPext));
      clientESP_RMS.stop();
      return;
    }
  }
  timeout = millis();
  // Lecture des données brutes distantes
  while (clientESP_RMS.available() && (millis() - timeout < 5000)) {
    RMSExtDataB += clientESP_RMS.readStringUntil('\r');
  }
  if (RMSExtDataB.length() > 300) {
    RMSExtDataB = "";
  }
  if (RMSExtDataB.indexOf("Deb") >= 0 && RMSExtDataB.indexOf("Fin") > 0) {  //Trame complète reçue
    RMSExtDataB = RMSExtDataB.substring(RMSExtDataB.indexOf("Deb") + 4);
    RMSExtDataB = RMSExtDataB.substring(0, RMSExtDataB.indexOf("Fin") + 3);
    String Sval = "";
    int idx = 0;
    while (RMSExtDataB.indexOf(GS) > 0) {
      Sval = RMSExtDataB.substring(0, RMSExtDataB.indexOf(GS));
      RMSExtDataB = RMSExtDataB.substring(RMSExtDataB.indexOf(GS) + 1);
      Gr[idx] = Sval;
      idx++;
    }
    Gr[idx] = RMSExtDataB;
    int Idx = 0;
    for (int i = 0; i < idx; i++) {
      while (Gr[i].indexOf(RS) >= 0) {
        Sval = Gr[i].substring(0, Gr[i].indexOf(RS));
        Gr[i] = Gr[i].substring(Gr[i].indexOf(RS) + 1);
        data_[Idx] = Sval;
        Idx++;
      }
      data_[Idx] = Gr[i];
      Idx++;
    }
    Imax = Idx;
    for (int i = 0; i < Idx; i++) {
      switch (i) {
        case 0:
          Heure = data_[i].substring(11, 19);
        case 1:
          Source_data = data_[i];
          break;
        case 2:
          LTARF = data_[i];
          break;
        case 3:
          STGE = data_[i];
          break;
        case 4:
          Temperature = data_[i].toFloat();
          break;
        case 5:
          Pva_valide = data_[i].toInt();
          break;
        case 6:
          PuissanceS_M = data_[i].toInt();
          break;
        case 7:
          PuissanceI_M = data_[i].toInt();
          break;
        case 8:
          PVAS_M = data_[i].toInt();
          break;
        case 9:
          PVAI_M = data_[i].toInt();
          break;
        case 10:
          EnergieJour_M_Soutiree = data_[i].toInt();
          break;
        case 11:
          EnergieJour_M_Injectee = data_[i].toInt();
          break;
        case 12:
          Energie_M_Soutiree = data_[i].toInt();
          break;
        case 13:
          Energie_M_Injectee = data_[i].toInt();
          break;
        case 14:  //CAS UxIx2 avec une deuxieme sonde
          PuissanceS_T = data_[i].toInt();
          break;
        case 15:
          PuissanceI_T = data_[i].toInt();
          break;
        case 16:
          PVAS_T = data_[i].toInt();
          break;
        case 17:
          PVAI_T = data_[i].toInt();
          break;
        case 18:
          EnergieJour_T_Soutiree = data_[i].toInt();
          break;
        case 19:
          EnergieJour_T_Injectee = data_[i].toInt();
          break;
        case 20:
          Energie_T_Soutiree = data_[i].toInt();
          break;
        case 21:
          Energie_T_Injectee = data_[i].toInt();
          break;
      }
    }
    for (int i = Idx + 1; i < Taille_data; i++) {
      data_[i] = "";
    }
    esp_task_wdt_reset();
    Bug_Host = 0;
    RMSExtDataB = "";
    clientESP_RMS.stop();
  }

  LastW = int(PuissanceS_M - PuissanceI_M);  //Watt value
  min10new = int(Heure.substring(3, 5).toInt() / 10);
  if (min10new != min10) {
    tabPw[idxPw] = LastW;
    min10 = min10new;
    idxPw = (1 + idxPw) % 240;
  }
}



void LireIR() {  //Capteur Infra Rouge
  int IR = digitalRead(GPIO_IR);
  if (IR == 1) {  //Présence source de chaleur en mouvement
    previousIRMillis = millis();
    if (previousIRMillis > 40000) GoodIRpulse = true;
    if (GoodIRpulse) AfficheurOn = true;
  } else {
    GoodIRpulse = true;
  }

  //MODIF TEST ****************
  //GoodIRpulse = true;
  //AfficheurOn = true;
  //previousIRMillis = millis();

  if (millis() - previousIRMillis > dureeAllumageMillis) {  //Attente On 20 secondes pour pouvoir dialoguer par internet
    Serial.println("Going to sleep now");                   // Mise en veille profonde
    tft.fillScreen(TFT_BLACK);
    esp_deep_sleep_start();
  }
}

//Sorties Afficheur
void Affichage() {
  if (WiFi.getMode() != WIFI_STA) {
    EcranInit1();
  } else {
    switch (idxDisp) {
      case 0:
        TraceWatt();  //Page principale
        break;
      case 1:
        Trace_Valeurs();
        break;
      case 2:
        TraceCourbe();
        break;
      case 3:
        TraceInfo();
        break;
    }
  }
}
void TraceWatt() {
  tft.setFreeFont(FSB24);
  tft.fillScreen(TFT_NAVY);
  TraceTarif();
  tft.drawCentreString(String(LastW) + " W", 120, 40, GFXFF);
  tft.setFreeFont(FS9);
  tft.drawString(Heure, 8, 3, 1);
  if (Temperature > -100) {
    String Tem = String(Temperature) + " C";
    tft.drawString(Tem, 120, 3, 1);
  }
  if (LastW > 0) {                        //Barre horizontale
    int WX = floor(LastW * 240 / 12000);  //Max conso 12kW
    tft.fillRect(0, 110, WX, 20, TFT_ORANGE);
  } else {
    int WX = floor(LastW * 240 / 5000);  // Pas la meme échelle pour production Max 5kW
    tft.fillRect(240 + WX, 110, -WX, 20, TFT_GREEN);
  }
}
void Trace_Valeurs() {
  tft.setFreeFont(FS9);
  tft.fillScreen(TFT_NAVY);
  TraceTarif();
  tft.drawString("Puissance Active : " + String(LastW) + " W", 8, 10, 1);
  tft.drawString("Soutire  : " + String(PVAS_M) + " VA", 8, 30, 1);  //Prend pas les accents
  tft.drawString("Injecte : " + String(PVAI_M) + " VA", 8, 50, 1);
  tft.drawString("Soutire ce jour : " + String(EnergieJour_M_Soutiree) + " Wh", 8, 70, 1);
  tft.drawString("Injecte ce jour : " + String(EnergieJour_M_Injectee) + " Wh", 8, 90, 1);
}
void TraceCourbe() {
  tft.fillScreen(TFT_NAVY);
  TraceTarif();
  int Pmin = -10;
  int Pmax = 10;
  int i;
  int heure;
  for (i = 0; i < 240; i++) {
    Pmin = min(Pmin, tabPw[i]);
    Pmax = max(Pmax, tabPw[i]);
  }
  Pmax = 1.05 * max(Pmax, -Pmin);
  Pmin = -Pmax;
  int idx = (idxPw + 1) % 240;
  ;
  int X0 = 0;
  int Y0 = map(tabPw[idx], Pmin, Pmax, 135, 0);
  for (i = 1; i < 240; i++) {
    int X1 = i;
    idx = (idx + 1) % 240;
    int Y1 = map(tabPw[idx], Pmin, Pmax, 135, 0);
    tft.drawLine(X0, Y0, X1, Y1, TFT_RED);
    X0 = X1;
    Y0 = Y1;
  }

  tft.setFreeFont(FS9);
  tft.setTextColor(TFT_RED, TFT_NAVY);
  tft.drawString("W", 8, 3, 1);
  tft.setTextColor(TFT_WHITE, TFT_NAVY);
  int X1;
  for (i = 0; i <= 4; i++) {
    heure = -i * 8;
    X0 = 240 - i * 48;
    X1 = X0 - 10;
    tft.drawString(String(heure) + "h", X1, 80, 1);
    tft.drawLine(X0, 67, X0, 70, TFT_WHITE);
  }
}
void TraceInfo() {
  tft.setFreeFont(FS9);
  tft.setTextColor(TFT_WHITE, TFT_NAVY);
  tft.fillScreen(TFT_NAVY);
  TraceTarif();
  tft.drawString("IP ESP32 cet affichage : ", 8, 10, 1);
  tft.drawString("  - " + WiFi.localIP().toString(), 8, 30, 1);
  tft.drawString("IP source externe  : ", 8, 55, 1);
  tft.drawString("  - " + IPext, 8, 75, 1);
  String S = "Niveau WIFI        : " + String(WiFi.RSSI()) + " dBm";
  tft.drawString(S, 8, 100, 1);
  S = "Version : "+ String(Version);
  tft.drawString(S, 8, 120, 1);
}
void TraceTarif() {
  String Tarif[5] = { "PLEINE", "CREUSE", "BLEU", "BLANC", "ROUGE" };
  unsigned int couleur[5] = { TFT_RED, TFT_GREEN, TFT_BLUE, TFT_WHITE, TFT_RED };
  int i;
  if (LTARF != "") {
    for (i = 0; i < 5; i++) {
      if (LTARF.indexOf(Tarif[i]) >= 0) {
        tft.fillRect(220, 0, 20, 20, couleur[i]);
      }
    }
  }

  long lendemain = int(strtol(STGE.c_str(), NULL, 16)) / 4;
  if (lendemain > 0) {
    tft.fillRect(230, 24, 10, 20, couleur[lendemain + 1]);
  }
}
void EcranInit1() {
  tft.setFreeFont(FS9);
  tft.setTextColor(TFT_WHITE, TFT_NAVY);
  tft.fillScreen(TFT_NAVY);
  TraceTarif();
  tft.drawString("Initialisation", 60, 10, 1);
  tft.drawString("Connectez vous au WIFI : ,", 8, 35, 1);
  tft.drawString("DISP-ESP32...", 60, 55, 1);
  tft.drawString("Dans un navigateur Web,", 8, 75, 1);
  tft.drawString("rentrez l'adresse IP :", 8, 95, 1);
  tft.drawString("192.168.4.1,", 60, 115, 1);
}