// ***************
// *  WEB SERVER *
// ***************
void Init_Server() {
  //Init Web Server on port 80
  server.on("/", handleRoot);
  server.on("/ajax_data", handle_ajax_data);
  server.on("/Para", handlePara);
  server.on("/ParaAjax", handleParaAjax);
  server.on("/ParaUpdate", handleParaUpdate);
  server.on("/restart", handleRestart);
  server.on("/AP_ScanWifi", handleAP_ScanWifi);
  server.on("/AP_SetWifi", handleAP_SetWifi);
  server.onNotFound(handleNotFound);
  server.begin();
  Serial.println("HTTP server started");
  
}


void handleRoot() {                  //Pages principales
  if (WiFi.getMode() != WIFI_STA) {  // en AP et STA mode
    server.send(200, "text/html", String(ConnectAP_Html));
  } else {  //Station Mode seul
    server.send(200, "text/html", String(MainHtml));
  }
  previousIRMillis = millis();
  dureeAllumageMillis=200000;
}
void handle_ajax_data() {
  String S = "";
  for (int i = 0; i < Imax; i++) {
    S += data_[i] + RS;
  }
  server.send(200, "text/html", S);
  previousIRMillis = millis();
  dureeAllumageMillis=20000;
}
void handlePara() {
  server.send(200, "text/html", String(ParaHtml));
  previousIRMillis = millis();
  dureeAllumageMillis=200000;
}
void handleParaAjax() {
  String S = String(RMSextIP) + RS + String(hostname) + RS + String(Version) + GS;
  int T = int(millis() / 36000);
  float H = float(T) / 100;
  S += String(H) + RS + WiFi.RSSI() + RS + WiFi.BSSIDstr() + RS + WiFi.macAddress() + RS + ssid + RS + WiFi.localIP().toString() + RS + WiFi.gatewayIP().toString() + RS + WiFi.subnetMask().toString();
  server.send(200, "text/html", String(S));
  previousIRMillis = millis();
  dureeAllumageMillis=200000;
}
void handleParaUpdate() {
  String lesparas = server.arg("lesparas");
  RMSextIP = strtoul(lesparas.c_str(), NULL, 10);
  EcritureEnROM();
  for (int i = 0; i < Taille_data; i++) {
    data_[i] = "";
  }
  server.send(200, "text/plain", "OK");
  previousIRMillis = millis();
  dureeAllumageMillis=200000;
}

void handleAP_ScanWifi() {
  Bug_Host = 0;
  Serial.println("Scan start");

  // WiFi.scanNetworks will return the number of networks found.
  int n = WiFi.scanNetworks();
  Serial.println("Scan done");
  String S = "";
  if (n == 0) {
    Serial.println("Pas de réseau Wifi trouvé");
  } else {
    Serial.print(n);
    Serial.println(" réseaux trouvés");
    Serial.println("Nr | SSID                             | RSSI | CH | Encryption");
    for (int i = 0; i < n; ++i) {
      // Print SSID and RSSI for each network found
      Serial.printf("%2d", i + 1);
      Serial.print(" | ");
      Serial.printf("%-32.32s", WiFi.SSID(i).c_str());
      Serial.print(" | ");
      Serial.printf("%4d", WiFi.RSSI(i));
      Serial.println();
      S += WiFi.SSID(i).c_str() + RS + WiFi.RSSI(i) + GS;
    }
  }
  WiFi.scanDelete();
  server.send(200, "text/html", S);
  previousIRMillis = millis();
  dureeAllumageMillis=300000;
}
void handleAP_SetWifi() {
  Bug_Host = 0;
  Serial.println("Set Wifi");
  String NewSsid = server.arg("ssid");
  NewSsid.trim();
  String NewPassword = server.arg("passe");
  NewPassword.trim();
  Serial.println(NewSsid);
  Serial.println(NewPassword);
  ssid = NewSsid;
  password = NewPassword;
  Serial.println("Wifi Begin : " + ssid);
  WiFi.begin(ssid.c_str(), password.c_str());
  unsigned long newstartMillis = millis();
  while (WiFi.status() != WL_CONNECTED && (millis() - newstartMillis < 15000)) {  // Attente connexion au Wifi
    Serial.write('.');
    Serial.print(WiFi.status());
    delay(300);
  }
  String S = "";
  if (WiFi.status() == WL_CONNECTED) {
    Serial.print("IP address: ");
    Serial.println(WiFi.localIP());
    String IP = WiFi.localIP().toString();
    S = "Ok" + RS;
    S += "ESP 32 connecté avec succès au wifi : " + ssid + " avec l'adresse IP : " + IP;
    S += "<br><br> Connectez vous au wifi : " + ssid;
    S += "<br><br> Cliquez sur l'adresse : <a href='http://" + IP + "' >http://" + IP + "</a>";
    EcritureEnROM();
  } else {
    S = "No" + RS + "ESP32 non connecté à :" + ssid + "<br>";
  }
  server.send(200, "text/html", S);
  delay(1000);
  ESP.restart();
  previousIRMillis = millis();
  dureeAllumageMillis=200000;
}

void handleNotFound() {  //Page Web pas trouvé
  String message = "Fichier non trouvé\n\n";
  message += "URI: ";
  message += server.uri();
  message += "\nMethod: ";
  message += (server.method() == HTTP_GET) ? "GET" : "POST";
  message += "\nArguments: ";
  message += server.args();
  message += "\n";
  for (uint8_t i = 0; i < server.args(); i++) {
    message += " " + server.argName(i) + ": " + server.arg(i) + "\n";
  }
  server.send(404, "text/plain", message);
}
void handleRestart() {  // Eventuellement Reseter l'ESP32 à distance
  server.send(200, "text/plain", "OK Reset. Attendez.");
  delay(1000);
  ESP.restart();
}